#!/bin/ksh
##############################################################################
# IBM Confidential
# OCO Source Materials
# 5724-S51
#
# (C) Copyright IBM Corp. 2008  All Rights Reserved.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
##############################################################################
#
##############################################################################
# Usage - Display usage statement to the screen.
##############################################################################
#
Usage()
{
   [[ " $NV_TRACE " = +(* all *|* install *) || -n $DEBUG ]] && set -x
#
   echo "Usage:  $SCRIPT  -i | -r"
   echo ""
   echo "where:"
   echo "  -i        Install InterimFix."
   echo "  -r        Rollback (uninstall) InterimFix."
   echo ""
   exit 0
}
#
##############################################################################
# ProcessFix - install a specific fix, holding output to display on error
##############################################################################
#
ProcessFix()
{
   cd $HERE/ifixes/$3/META-INF
#
   if [ "$1" = "update" ]; then
      options="-u true"
      printf "Installing $3...\n"
   else
      printf "Uninstalling $3...\n"
   fi
#
   manageIU.sh -o $1 -r $2 -p packagedIU.xml $options 1>>$LOGFILE 2>&1
   ReturnCode=$?
#
   if [ $ReturnCode -ne 0 ]; then
      ERROR=3
      if [ "$1" = "update" ]; then
         printf "Installation of $3 failed.\n"
      else
         printf "Uninstallation of $3 failed.\n"
      fi
   fi
#
   cd - 1>/dev/null 2>&1
   return $ReturnCode
}
#
##############################################################################
# Main
##############################################################################
#
#------------------------------------------------------------------------
# Set up local variables and constants
#------------------------------------------------------------------------
#
SCRIPT=`basename $0`
LOGFILE=/tmp/IBM_Tivoli_Netcool_Webtop_InterimFix.log
ERROR=0
DEFAULT_INSTALLATION_DIR=/opt/IBM/tivoli/tip
#
cd `dirname $0`
HERE=`pwd`
cd - 1>/dev/null 2>&1
#
echo "IBM Tivoli Netcool/Webtop 2.2 InterimFix 002"
#
#-----------------------------------------------------------------------
# Parse the command line arguments
#-----------------------------------------------------------------------
#
while [ $OPTIND -le $# ]
do
   [ $OPTIND -gt $# ] && break
#
   while getopts :id:r c
   do
      case $c in
        i)      DO=update;;
        r)      DO=undo;;
        *)      Usage
                exit 1;;
      esac
   done
done
#
if [ -z "$DO" ]; then
   Usage
   exit 1
fi
#
#-----------------------------------------------------------------------
# Get the location of the TIP installation on this system
#-----------------------------------------------------------------------
#
while [ ! -x $TIP_INSTALL/bin/startServer.sh ]
do
   printf "Please enter path to Tivoli Integrated Portal installation directory [$DEFAULT_INSTALLATION_DIR]"
   read TIP_INSTALL
   [ "$TIP_INSTALL" = "" ] && TIP_INSTALL=$DEFAULT_INSTALLATION_DIR
done
#
cd $TIP_INSTALL
TIP_INSTALL=`pwd`
cd - 1>/dev/null 2>&1
#
#-----------------------------------------------------------------------
# Set up the DE environment
#-----------------------------------------------------------------------
#
printf "Setting up Deployment Engine environment.\n"
#
if [ -x /var/ibm/common/acsi/setenv.sh ]; then
   . /var/ibm/common/acsi/setenv.sh
elif [ -x ~/.acsi_$LOGNAME/setenv.sh ]; then
   . ~/.acsi_$LOGNAME/setenv.sh
else
   echo "$SCRIPT: Unable to set up Deployment Engine environment for this user."
   exit 1
fi
#
if [ $DO = "update" ]; then
   printf "Installing Deployment Engine...\n"
   $HERE/DE/si_inst.sh 1>>$LOGFILE 2>&1
fi
#
#-----------------------------------------------------------------------
# Process the patches as requested by the user on the command line
#-----------------------------------------------------------------------
#
if [ -e $TIP_INSTALL/profiles/TIPProfile/logs/server1/server1.pid ]; then
   if [ $DO = "update" ]; then
      printf "Please stop Tivoli Integrated Portal before installing InterimFix.\n"
   else
      printf "Please stop Tivoli Integrated Portal before uninstalling InterimFix.\n"
   fi
   exit 1
fi
#
ProcessFix $DO $TIP_INSTALL CommonReporting-TIP-1200-CommonReporting-0001
ProcessFix $DO $TIP_INSTALL ISC-TIP-7500-ISC-0003
ProcessFix $DO $TIP_INSTALL ISC-TIP-7500-ISC-0004
#ProcessFix $DO $TIP_INSTALL ITMWebService-TIP-1100-ITMWebService-0002
ProcessFix $DO $TIP_INSTALL TIP-TIP-1100-TIP-0002
#ProcessFix $DO $TIP_INSTALL TIPChart-TIP-1100-TIPChart-0002
ProcessFix $DO $TIP_INSTALL Webtop-22082-InterimFix002
#
if [ $ERROR = 0 ]; then
   if [ $DO = "update" ]; then
      printf "InterimFix installed successfully.\n"
   else
      printf "InterimFix uninstalled successfully.\n"
   fi
   printf "Please start Tivoli Integrated Portal.\n"
else
   if [ $DO = "update" ]; then
      printf "InterimFix installation failed.\n"
   else
      printf "InterimFix uninstallation failed.\n"
   fi
   printf "Check details in $LOGFILE\n"
fi
#
##############################################################################
# End of Program
##############################################################################
#
exit $ERROR
