/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: %W%
 * Version:       %I%
 * Last Modified: %G%
 *******************************************************************************/

dojo.provide("ibm.tivoli.tip.charts.dijit.ParametersWidget");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.form.TextBox");

// NLS
dojo.requireLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");


// define the widget class
dojo.declare (
	"ibm.tivoli.tip.charts.dijit.ParametersWidget", 
    [ dijit._Widget, dijit._Templated, dijit._Container ],
{
    namespace: "namespace1_",
    widgetsInTemplate: true,
    data: [],
    context: [],
    hasContext_: false,
	templatePath: dojo.moduleUrl("ibm.tivoli.tip.charts.dijit","templates/ParametersWidget.html"),

	constructor: function() {
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
	    this.resourceMessages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
	},

	postCreate: function() { 
		this.inherited ("postCreate", arguments);
		console.info ("enter ParametersWidget:postCreate");
//		console.log ("looking for div: " + this.id);
//		var mydiv = dijit.byId (this.id);
//		console.info (mydiv);
		console.log ("ParametersWidget.domNode: ");
		console.log (this.domNode);
//		this.domNode.innerHTML = this.getContent(this.data);
		this.addParameters (this.data);
	},
	
	addParameters: function (model) {
		console.info ("enter ParametersWidget.addParameters");
		var param = null;
		var widget = null;

		for (var row in model) {
			if (row) {
				param = model[row];
				widget = new ibm.tivoli.tip.charts.dijit.Parameter ({
					parmid: this.namespace + "param_" + param.name,
					baseid: param.name,
					value: param.value,
					label: param.name
				});
				
				this.addChild (widget);
			}
		}
	},
	
	getContent: function(model) {
		console.info ("enter ParametersWidget.getContent");
		var innerHTML = "";

		innerHTML += "<table>";
		
		innerHTML += "<tr>";
		innerHTML += "<td><h2>";
		innerHTML += this.resourceMessages_.DEFAULT_PARAMETERS;
		innerHTML += "</h2></td>";
		innerHTML += "</tr>";
		
		var param = null;

		for (var row in model) {
			if (row) {
				param = model[row];

				innerHTML += "<tr>";
				innerHTML += "<td><div class=\"label\">";
				innerHTML += param.name;
			
				innerHTML += "</div></td>";
				innerHTML += "<td><div><input id=\"";
				innerHTML += this.namespace;
				innerHTML += "param_";
				innerHTML += param.name;
				innerHTML += "\" type=\"text\" name=\"";
				innerHTML += this.namespace;
				innerHTML += "param_";
				innerHTML += param.name;
				innerHTML += "\" value=\"";
				if (param.value) {
					innerHTML += param.value;
				}
				innerHTML += "\" dojoType=\"TextBox\">";
				innerHTML += "</input></div>";
				innerHTML += "</td>";
				innerHTML += "</tr>";
			}
		}
		
		innerHTML += "</table>";

		console.log ("return innerHTML:");
		console.log (innerHTML);
		return innerHTML;
	},
	
	isContextOn: function () {
		var receiveContextRadio = dijit.byId(this.namespace + "contextualParametersSelected");
		console.log ("isContextOn: " + receiveContextRadio.checked);
		return receiveContextRadio.checked;
	},
	
	hasContext: function() {
		return this.hasContext_;
	},
	
	getParameterValue: function (parameterName) {
		console.info ("enter ParametersWidget.getParameterValue");
	
		var value = null;
		
		if (this.isContextOn() && this.hasContext()) {
			console.log ("context is on and we have context");
			if (this.context[parameterName]) {
				console.log ("from context w/ parameter: " + parameterName + " and value: " + this.context[parameterName]);
				value = this.context[parameterName];
			} else {
				console.log ("context value is null for name: " + parameterName);
				value = null;
			}
		} else {
			var textbox = dijit.byId (this.namespace + "param_" + parameterName);
			console.log ("getting param value from default value: ");
			console.log (textbox);
			value = textbox.getValue();
		}
		
		value = this.urlizeValue (value);
		
		return (value);
	},
	
	urlizeValue: function (value) {
		if (value !== null) {
			value = value.replace ("&", ":::");
		}
		
		return (value);
	},

	setParameterValue: function (parameterName, parameterValue) {
		console.info ("enter ParametersWidget.setParameterValue");
		var textbox = dijit.byId (this.namespace + "param_" + parameterName);
		console.log ("ParametersWidget.setParameterValue, textbox: ");
		console.log (textbox);
		textbox.setValue (parameterValue);
	},

	setAllContext: function (ctxt) {
		var params = this.getParameters();
		var param = null;
		var value = null;
		var x = null;
		
		for (x in params) {
			if (x) {
				param = params[x];
				value = eval ("ctxt." + param.name);
				if (value) {
					this.setContext (param.name, value);
				}
			}
		}
		
		this.hasContext_ = true;
	},
	
	setContext: function (parameterName, parameterValue) {
		this.context[parameterName] = parameterValue;
		this.hasContext_ = true;
	},
	
	getParameters: function () {
		return this.data;
	},
	
	updateInternalParametersData: function () {
		dojo.forEach (this.data, function (datum) {
			var param = dijit.byId (this.namespace + "param_" + datum.name);
			datum.value = param.getValue();
		}, this);
	},
	
	removeParameters: function () {
		console.log ("enter ParametersWidget.removeParameters");
		dojo.forEach (this.getChildren(), function (child) {
			this.removeChild (child);
			child.destroy();
		}, this);
		console.log ("exit ParametersWidget.removeParameters");
	},
	
	resetParameters: function () {
		console.log ("enter ParametersWidget.resetParameters");
		var param = null;
		var widget = null;
		for (var row in this.data) {
			if (row) {
				console.log ("resetting for row: " + row);
				param = this.data[row];
				widget = dijit.byId (this.namespace + "param_" + param.name);
				if (param.value === undefined) {
					param.value = "";
				}
				widget.setValue (param.value);
			}
		}

		console.log ("exit ParametersWidget.resetParameters");
	},
	
	destroy: function () {
		console.log ("Enter ParametersWidget.destroy");
//		this.removeParameters();
		dojo.forEach (this.data, function (datum) {
			var param = dijit.byId (this.namespace + "param_" + datum.name);
			if (param) {
				param.destroy();
			}
		}, this);
		
		this.data = null;
		this.destroyDescendants();
		this.inherited (arguments);
		console.log ("Exit ParametersWidget.destroy");
	}

});
	

// define the widget class
dojo.declare (
	"ibm.tivoli.tip.charts.dijit.Parameter", 
    [ dijit._Widget, dijit._Templated ],
{
	parmid: null,
	baseid: null,
	label: null,
	description: null,
	value: null,
	widgetsInTemplate: true,
	templateString: '<div>' +
		'<label for="${id}">${label}</label> ' +
		'<input type="text" dojoType="dijit.form.TextBox" dojoAttachPoint="textbox" name="${parmid}" id="${parmid}" value="" />' +
		'</div><br/>',
	
	constructor: function () {
	},
	postCreate: function () {
		if (this.value) {
			this.textbox.setValue (this.value);
		}
	},
	setValue: function (value) {
		this.textbox.setValue (value);
	},
	getValue: function () {
		return this.textbox.getValue();
	},
	destroy: function () {
		console.log ("enter Parameter.destroy for");
		console.log (this);
//		if (dijit.byId(this.parmid)) {
//			console.log ("about to destroy: ");
//			console.log (dijit.byId (this.parmid));
//			dijit.byId (this.parmid).destroy();
//		} else {
//			console.log ("not found to destroy: " + this.parmid);
//		}

//		this.destroyDescendants();
		this.inherited (arguments);
		console.log ("exit Parameter.destroy");
	}
	
});

